@@ -12,26 +12,26 @@
 # You should have received a copy of the GNU General Public License
 # along with torf.  If not, see <https://www.gnu.org/licenses/>.
 
-import flatbencode as bencode
 import base64
-import hashlib
-from datetime import datetime
-import os
-import math
-from collections import abc, namedtuple
 import errno
+import hashlib
 import inspect
 import io
-import pathlib
-import fnmatch
-import re
 import itertools
+import math
+import os
+import pathlib
+import re
+from collections import abc
+from datetime import datetime
 
-from . import _utils as utils
-from . import _errors as error
-from . import _generate as generate
+import flatbencode as bencode
 
 from . import __version__
+from . import _errors as error
+from . import _generate as generate
+from . import _utils as utils
+
 _PACKAGE_NAME = __name__.split('.')[0]
 
 # os.sched_getaffinity() is only available on some Unix platforms.
@@ -70,9 +70,9 @@ class Torrent():
 
     Hash pieces and update progress once per second:
 
-    >>> def callback(torrent, filepath, pieces_done, pieces_total):
+    >>> def cb(torrent, filepath, pieces_done, pieces_total):
     ...     print(f'{pieces_done/pieces_total*100:3.0f} % done')
-    >>> success = torrent.generate(callback, interval=1)
+    >>> success = torrent.generate(callback=cb, interval=1)
       1 % done
       2 % done
       [...]
@@ -159,6 +159,7 @@ class Torrent():
             readable
         """
         return self._path
+
     @path.setter
     def path(self, value):
         if value is None:
@@ -209,7 +210,6 @@ class Torrent():
 
     @files.setter
     def files(self, files):
-        info = self.metainfo['info']
         if not isinstance(files, utils.Iterable):
             raise ValueError(f'Not an Iterable: {files}')
         for f in files:
@@ -265,7 +265,6 @@ class Torrent():
         if not isinstance(filepaths, utils.Iterable):
             raise ValueError(f'Not an Iterable: {filepaths}')
 
-        info = self.metainfo['info']
         filepaths = utils.Filepaths(filepaths)  # Resolve directories
         if not filepaths:
             self._set_files(files=())
@@ -380,6 +379,7 @@ class Torrent():
         ======== ============================
         """
         return self._exclude['globs']
+
     @exclude_globs.setter
     def exclude_globs(self, value):
         if not isinstance(value, utils.Iterable):
@@ -396,6 +396,7 @@ class Torrent():
         :raises re.error: if any regular expression is invalid
         """
         return self._exclude['regexs']
+
     @exclude_regexs.setter
     def exclude_regexs(self, value):
         if not isinstance(value, utils.Iterable):
@@ -461,6 +462,7 @@ class Torrent():
         if 'name' not in self.metainfo['info'] and self.path is not None:
             self.metainfo['info']['name'] = self.path.name
         return self.metainfo['info'].get('name', None)
+
     @name.setter
     def name(self, value):
         if value is None:
@@ -515,7 +517,7 @@ class Torrent():
         elif self.mode == 'multifile':
             file_sizes = []
             for info in self.metainfo['info']['files']:
-                this_path = (self.name,) + tuple(info['path'])
+                this_path = (self.name,) + tuple(c for c in info['path'] if c)
                 if this_path == path:
                     # path points to file
                     return info['length']
@@ -540,6 +542,7 @@ class Torrent():
         if 'piece length' not in self.metainfo['info']:
             self.piece_size = None  # Calculate piece size
         return self.metainfo['info'].get('piece length', 0)
+
     @piece_size.setter
     def piece_size(self, value):
         if value is None:
@@ -577,13 +580,13 @@ class Torrent():
 
         :return: calculated piece size
         """
-        if size <= 2**30:          #  1 GiB / 1024 pieces = 1 MiB max
+        if size <= 2**30:          # 1 GiB / 1024 pieces = 1 MiB max
             pieces = size / 1024
-        elif size <= 4 * 2**30:    #  4 GiB / 2048 pieces = 2 MiB max
+        elif size <= 4 * 2**30:    # 4 GiB / 2048 pieces = 2 MiB max
             pieces = size / 2048
-        elif size <= 6 * 2**30:    #  6 GiB / 3072 pieces = 2 MiB max
+        elif size <= 6 * 2**30:    # 6 GiB / 3072 pieces = 2 MiB max
             pieces = size / 3072
-        elif size <= 8 * 2**30:    #  8 GiB / 2048 pieces = 4 MiB max
+        elif size <= 8 * 2**30:    # 8 GiB / 2048 pieces = 4 MiB max
             pieces = size / 2048
         elif size <= 16 * 2**30:   # 16 GiB / 2048 pieces = 8 MiB max
             pieces = size / 2048
@@ -606,7 +609,7 @@ class Torrent():
     :class:`PieceSizeError`.
     """
 
-    piece_size_max = 16 * 1024*1024  # 16 MiB
+    piece_size_max = 16 * 1024 * 1024  # 16 MiB
     """
     Greatest allowed piece size
 
@@ -630,7 +633,7 @@ class Torrent():
         hashes = self.metainfo['info'].get('pieces')
         if isinstance(hashes, (bytes, bytearray)):
             # Each hash is 20 bytes long
-            return tuple(bytes(hashes[pos:pos+20])
+            return tuple(bytes(hashes[pos : pos + 20])
                          for pos in range(0, len(hashes), 20))
         else:
             return ()
@@ -776,12 +779,15 @@ class Torrent():
         """
         Whether torrent should use trackers exclusively for peer discovery
 
-        Setting this property to ``None`` removes ``private`` from
-        :attr:`metainfo`\ ``['info']``.  Otherwise,
-        :attr:`metainfo`\ ``['info']``\ ``['private']`` is set to a
-        :class:`bool`.
+        ``True`` or ``False`` if
+        :attr:`metainfo`\ ``['info']``\ ``['private']`` exists, ``None``
+        otherwise.
         """
-        return bool(self.metainfo['info'].get('private', False))
+        if 'private' in self.metainfo['info']:
+            return bool(self.metainfo['info']['private'])
+        else:
+            return None
+
     @private.setter
     def private(self, value):
         if value is None:
@@ -797,6 +803,7 @@ class Torrent():
         Setting this property sets or removes :attr:`metainfo`\ ``['comment']``.
         """
         return self.metainfo.get('comment', None)
+
     @comment.setter
     def comment(self, value):
         if value is not None:
@@ -816,6 +823,7 @@ class Torrent():
         :attr:`metainfo`\ ``['creation date']``.
         """
         return self.metainfo.get('creation date', None)
+
     @creation_date.setter
     def creation_date(self, value):
         if isinstance(value, (float, int)):
@@ -836,6 +844,7 @@ class Torrent():
         :attr:`metainfo`\ ``['created by']``.
         """
         return self.metainfo.get('created by', None)
+
     @created_by.setter
     def created_by(self, value):
         if value is not None:
@@ -852,6 +861,7 @@ class Torrent():
         :attr:`metainfo`\ ``['info']``\ ``['created by']``.
         """
         return self.metainfo['info'].get('source', None)
+
     @source.setter
     def source(self, value):
         if value is not None:
@@ -901,6 +911,7 @@ class Torrent():
         Setting it to ``False`` removes that field.
         """
         return bool(self.metainfo['info'].get('entropy', False))
+
     @randomize_infohash.setter
     def randomize_infohash(self, value):
         if value:
@@ -969,7 +980,7 @@ class Torrent():
         # Read piece_size'd chunks from disk and push them to queue for hashing
         reader = generate.Reader(filepaths=filepaths,
                                  piece_size=self.piece_size,
-                                 queue_size=threads*3)
+                                 queue_size=threads * 3)
 
         # Pool of workers that pull from reader's piece queue, calculate the
         # hashes, and quickly offload the results to a hash queue
@@ -988,9 +999,9 @@ class Torrent():
                                               callback=collector_callback)
 
         if maybe_cancel:
-           maybe_cancel.on_cancel(reader.stop,
-                                  hasher_threadpool.stop,
-                                  collector_thread.stop)
+            maybe_cancel.on_cancel(reader.stop,
+                                   hasher_threadpool.stop,
+                                   collector_thread.stop)
 
         try:
             reader.read()
@@ -1186,7 +1197,7 @@ class Torrent():
                                      file_sizes={fs_path:self.partial_size(t_path)
                                                  for fs_path,t_path in filepaths},
                                      piece_size=self.piece_size,
-                                     queue_size=threads*3,
+                                     queue_size=threads * 3,
                                      skip_on_error=skip_on_error)
 
             # Pool of workers that pull from reader_thread's piece queue, calculate
@@ -1256,9 +1267,9 @@ class Torrent():
         Check if all mandatory keys exist in :attr:`metainfo` and all standard keys
         have correct types
 
-        The necessary values are documented here:
-        http://bittorrent.org/beps/bep_0003.html
-        https://wiki.theory.org/index.php/BitTorrentSpecification#Metainfo_File_Structure
+        References:
+          | http://bittorrent.org/beps/bep_0003.html
+          | https://wiki.theory.org/index.php/BitTorrentSpecification#Metainfo_File_Structure
 
         :raises MetainfoError: if :attr:`metainfo` would not generate a valid
             torrent file or magnet link
@@ -1267,6 +1278,7 @@ class Torrent():
         info = md['info']
 
         # Check values shared by singlefile and multifile torrents
+        utils.assert_type(md, ('info',), (dict,), must_exist=True)
         utils.assert_type(md, ('info', 'name'), (str,), must_exist=True)
         utils.assert_type(md, ('info', 'piece length'), (int,), must_exist=True,
                           check=utils.is_power_of_2)
@@ -1469,6 +1481,10 @@ class Torrent():
         :param bool validate: Whether to run :meth:`validate` on the new Torrent
             instance
 
+            NOTE: If the "info" field is not a dictionary,
+                  :class:`MetainfoError` is raised even if `validate` is set to
+                  False
+
         :raises ReadError: if reading from `stream` fails
         :raises BdecodeError: if `stream` does not produce a valid bencoded byte
             sequence
@@ -1484,29 +1500,28 @@ class Torrent():
         else:
             try:
                 metainfo_enc = bencode.decode(content)
-            except bencode.DecodingError:
+            except (bencode.DecodingError, ValueError):
                 raise error.BdecodeError()
             else:
                 if not isinstance(metainfo_enc, abc.Mapping):
                     raise error.BdecodeError()
 
-            if validate:
-                if b'info' not in metainfo_enc:
-                    raise error.MetainfoError("Missing 'info'")
-                elif not isinstance(metainfo_enc[b'info'], abc.Mapping):
-                    raise error.MetainfoError("'info' is not a dictionary")
-                elif b'pieces' not in metainfo_enc[b'info']:
-                    raise error.MetainfoError("Missing 'pieces' in ['info']")
-
-            # Extract 'pieces' from metainfo because it's the only byte sequence
-            # that isn't supposed to be decoded to unicode.
-            if b'info' in metainfo_enc and b'pieces' in metainfo_enc[b'info']:
+            # Extract 'pieces' from metainfo before decoding because it's the
+            # only byte sequence that isn't supposed to be decoded to a string.
+            if (b'info' in metainfo_enc and
+                isinstance(metainfo_enc[b'info'], dict) and
+                b'pieces' in metainfo_enc[b'info']):
                 pieces = metainfo_enc[b'info'].pop(b'pieces')
                 metainfo = utils.decode_dict(metainfo_enc)
                 metainfo['info']['pieces'] = pieces
             else:
                 metainfo = utils.decode_dict(metainfo_enc)
 
+            # "info" must be a dictionary.  If validation is not wanted, it's OK
+            # if it doesn't exist because the "metainfo" property will add
+            # automatically.
+            utils.assert_type(metainfo, ('info',), (dict,), must_exist=validate)
+
             torrent = cls()
             torrent._metainfo = metainfo
 
