@@ -12,19 +12,24 @@
 # You should have received a copy of the GNU General Public License
 # along with torf.  If not, see <https://www.gnu.org/licenses/>.
 
-import os
-import math
-import fnmatch
-from urllib.parse import urlparse
-from urllib.parse import quote_plus as urlquote
-import collections
 import abc
-import re
-import errno
-from datetime import datetime
-import itertools
+import collections
 import contextlib
+import errno
+import fnmatch
+import http.client
+import itertools
+import math
+import os
 import pathlib
+import re
+import socket
+import typing
+import urllib.error
+import urllib.parse
+import urllib.request
+from datetime import datetime
+from urllib.parse import quote_plus as urlquote  # noqa: F401
 
 from . import _errors as error
 
@@ -92,10 +97,11 @@ def real_size(path):
     Raise ReadError on failure
     """
     if os.path.isdir(os.path.realpath(path)):
-        size = 0
         def onerror(exc):
             raise error.ReadError(getattr(exc, 'errno', None),
                                   getattr(exc, 'filename', None))
+
+        size = 0
         walker = os.walk(path, followlinks=True, onerror=onerror)
         for dirpath,dirnames,filenames in walker:
             for filename in filenames:
@@ -156,7 +162,7 @@ def filter_files(items, getter=lambda f: f, exclude=(), hidden=True, empty=True)
         return False
 
     def is_excluded(path,
-                    regexs=tuple(x for x in exclude if isinstance(x, re.Pattern)),
+                    regexs=tuple(x for x in exclude if isinstance(x, typing.Pattern)),
                     globs=tuple(x for x in exclude if isinstance(x, str))):
         if any(r.search(str(path)) for r in regexs):
             return True
@@ -315,7 +321,7 @@ class File(os.PathLike):
         return str(self._path)
 
     def __eq__(self, other):
-        if isinstance(other, type(self)):
+        if type(other) is type(self):
             return self._path == other._path and self._size == other._size
         elif isinstance(other, os.PathLike):
             return self._path == other
@@ -434,7 +440,7 @@ class Filepaths(MonitoredList):
 def is_url(url):
     """Return whether `url` is a valid URL"""
     try:
-        u = urlparse(url)
+        u = urllib.parse.urlparse(url)
         u.port  # Trigger 'invalid port' exception
     except Exception:
         return False
@@ -444,10 +450,31 @@ def is_url(url):
         return True
 
 class URL(str):
+    def __new__(cls, s):
+        return super().__new__(cls, str(s).replace(' ', '+'))
+
     def __init__(self, url):
-        url = str(url)
         if not is_url(url):
             raise error.URLError(url)
+        else:
+            self._parsed = urllib.parse.urlparse(url)
+
+    @property
+    def scheme(self): return self._parsed.scheme
+    @property
+    def netloc(self): return self._parsed.netloc
+    @property
+    def hostname(self): return self._parsed.hostname
+    @property
+    def port(self): return self._parsed.port
+    @property
+    def path(self): return self._parsed.path
+    @property
+    def params(self): return self._parsed.params
+    @property
+    def query(self): return self._parsed.query
+    @property
+    def fragment(self): return self._parsed.fragment
 
 class URLs(MonitoredList):
     """Auto-flattening list of `:class:URL` objects with change callback"""
@@ -576,6 +603,42 @@ class Trackers(collections.abc.MutableSequence):
         return repr(self._tiers)
 
 
+def download(url, timeout=60):
+    """
+    Download data from URL
+
+    :raises ConnectionError: if the download fails
+
+    :return: the downloaded data or ``None`` if the protocol is not supported
+    """
+    if timeout <= 0:
+        raise error.ConnectionError(url, 'Timed out')
+    elif url.startswith('http://') or url.startswith('https://'):
+        return download_http(url, timeout=timeout)
+    else:
+        raise error.ConnectionError(url, 'Unsupported protocol')
+
+def download_http(url, timeout=60):
+    try:
+        response = urllib.request.urlopen(URL(url), timeout=timeout).read()
+    except urllib.error.URLError as e:
+        try:
+            msg = e.args[0].strerror
+        except (AttributeError, IndexError):
+            msg = (getattr(e, 'msg', None) or
+                   getattr(e, 'strerror', None) or
+                   'Failed')
+        raise error.ConnectionError(url, msg)
+    except socket.timeout:
+        raise error.ConnectionError(url, 'Timed out')
+    except http.client.HTTPException:
+        raise error.ConnectionError(url, 'No HTTP response')
+    except (OSError, IOError):
+        raise error.ConnectionError(url, 'Unknown error')
+    else:
+        return response
+
+
 class Iterable(abc.ABC):
     """
     Iterable that is not a :class:`str`
@@ -608,15 +671,15 @@ def key_exists_in_list_or_dict(key, lst_or_dct):
 
 def assert_type(obj, keys, exp_types, must_exist=True, check=None):
     """
-    Raise MetainfoError is not of a particular type
+    Raise MetainfoError value is not of a particular type
 
     :param obj: The object to check
     :type obj: sequence or mapping
     :param keys: Sequence of keys so that ``obj[key[0]][key[1]]...`` resolves to
         a value
     :type obj: sequence
-    :param exp_types: Sequence of types that the value specified by `keys` must
-        be an instance of
+    :param exp_types: Sequence of allowed types that the value specified by
+        `keys` must be an instance of
     :type obj: sequence
     :param bool must_exist: Whether to raise MetainfoError if `keys` does not
          resolve to a value
@@ -638,7 +701,10 @@ def assert_type(obj, keys, exp_types, must_exist=True, check=None):
 
     if not key_exists_in_list_or_dict(key, obj):
         if must_exist:
-            raise error.MetainfoError(f"Missing {key!r} in {keychain_str}")
+            if keychain_str:
+                raise error.MetainfoError(f'Missing {key!r} in {keychain_str}')
+            else:
+                raise error.MetainfoError(f'Missing {key!r}')
 
     elif not isinstance(obj[key], exp_types):
         if len(exp_types) > 2:
@@ -647,8 +713,8 @@ def assert_type(obj, keys, exp_types, must_exist=True, check=None):
         else:
             exp_types_str = ' or '.join(t.__name__ for t in exp_types)
         type_str = type(obj[key]).__name__
-        raise error.MetainfoError(f"{keychain_str}[{key!r}] must be {exp_types_str}, "
-                                  f"not {type_str}: {obj[key]!r}")
+        raise error.MetainfoError(f'{keychain_str}[{key!r}] must be {exp_types_str}, '
+                                  f'not {type_str}: {obj[key]!r}')
 
     elif check is not None and not check(obj[key]):
         raise error.MetainfoError(f"{keychain_str}[{key!r}] is invalid: {obj[key]!r}")
